# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    NumberOrderPhoneNumberListResponse,
    NumberOrderPhoneNumberRetrieveResponse,
    NumberOrderPhoneNumberUpdateRequirementsResponse,
    NumberOrderPhoneNumberUpdateRequirementGroupResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNumberOrderPhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.retrieve(
            "number_order_phone_number_id",
        )
        assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.number_order_phone_numbers.with_raw_response.retrieve(
            "number_order_phone_number_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = response.parse()
        assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.number_order_phone_numbers.with_streaming_response.retrieve(
            "number_order_phone_number_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = response.parse()
            assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `number_order_phone_number_id` but received ''"
        ):
            client.number_order_phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.list()
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.list(
            filter={"country_code": "US"},
        )
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.number_order_phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = response.parse()
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.number_order_phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = response.parse()
            assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_requirement_group(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_requirement_group(self, client: Telnyx) -> None:
        response = client.number_order_phone_numbers.with_raw_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = response.parse()
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_requirement_group(self, client: Telnyx) -> None:
        with client.number_order_phone_numbers.with_streaming_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = response.parse()
            assert_matches_type(
                NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_requirement_group(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.number_order_phone_numbers.with_raw_response.update_requirement_group(
                id="",
                requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_requirements(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_requirements_with_all_params(self, client: Telnyx) -> None:
        number_order_phone_number = client.number_order_phone_numbers.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_requirements(self, client: Telnyx) -> None:
        response = client.number_order_phone_numbers.with_raw_response.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = response.parse()
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_requirements(self, client: Telnyx) -> None:
        with client.number_order_phone_numbers.with_streaming_response.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = response.parse()
            assert_matches_type(
                NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_requirements(self, client: Telnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `number_order_phone_number_id` but received ''"
        ):
            client.number_order_phone_numbers.with_raw_response.update_requirements(
                number_order_phone_number_id="",
            )


class TestAsyncNumberOrderPhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.retrieve(
            "number_order_phone_number_id",
        )
        assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_order_phone_numbers.with_raw_response.retrieve(
            "number_order_phone_number_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = await response.parse()
        assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_order_phone_numbers.with_streaming_response.retrieve(
            "number_order_phone_number_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = await response.parse()
            assert_matches_type(NumberOrderPhoneNumberRetrieveResponse, number_order_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `number_order_phone_number_id` but received ''"
        ):
            await async_client.number_order_phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.list()
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.list(
            filter={"country_code": "US"},
        )
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_order_phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = await response.parse()
        assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_order_phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = await response.parse()
            assert_matches_type(NumberOrderPhoneNumberListResponse, number_order_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_order_phone_numbers.with_raw_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = await response.parse()
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_order_phone_numbers.with_streaming_response.update_requirement_group(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = await response.parse()
            assert_matches_type(
                NumberOrderPhoneNumberUpdateRequirementGroupResponse, number_order_phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_requirement_group(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.number_order_phone_numbers.with_raw_response.update_requirement_group(
                id="",
                requirement_group_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_requirements(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_requirements_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_order_phone_number = await async_client.number_order_phone_numbers.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_requirements(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_order_phone_numbers.with_raw_response.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order_phone_number = await response.parse()
        assert_matches_type(
            NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_requirements(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_order_phone_numbers.with_streaming_response.update_requirements(
            number_order_phone_number_id="number_order_phone_number_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order_phone_number = await response.parse()
            assert_matches_type(
                NumberOrderPhoneNumberUpdateRequirementsResponse, number_order_phone_number, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_requirements(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `number_order_phone_number_id` but received ''"
        ):
            await async_client.number_order_phone_numbers.with_raw_response.update_requirements(
                number_order_phone_number_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConferenceCreateParams"]


class ConferenceCreateParams(TypedDict, total=False):
    call_control_id: Required[str]
    """Unique identifier and token for controlling the call"""

    name: Required[str]
    """Name of the conference"""

    beep_enabled: Literal["always", "never", "on_enter", "on_exit"]
    """
    Whether a beep sound should be played when participants join and/or leave the
    conference.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string. The client_state will be updated for
    the creator call leg and will be used for all webhooks related to the created
    conference.
    """

    comfort_noise: bool
    """Toggle background comfort noise."""

    command_id: str
    """Use this field to avoid execution of duplicate commands.

    Telnyx will ignore subsequent commands with the same `command_id` as one that
    has already been executed.
    """

    duration_minutes: int
    """Time length (minutes) after which the conference will end."""

    hold_audio_url: str
    """The URL of a file to be played to participants joining the conference.

    The URL can point to either a WAV or MP3 file. hold_media_name and
    hold_audio_url cannot be used together in one request. Takes effect only when
    "start_conference_on_create" is set to "false".
    """

    hold_media_name: str
    """The media_name of a file to be played to participants joining the conference.

    The media_name must point to a file previously uploaded to
    api.telnyx.com/v2/media by the same user/organization. The file must either be a
    WAV or MP3 file. Takes effect only when "start_conference_on_create" is set to
    "false".
    """

    max_participants: int
    """The maximum number of active conference participants to allow.

    Must be between 2 and 800. Defaults to 250
    """

    start_conference_on_create: bool
    """Whether the conference should be started on creation.

    If the conference isn't started all participants that join are automatically put
    on hold. Defaults to "true".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ShortCodeListParams", "Filter", "Page"]


class ShortCodeListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[messaging_profile_id]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class Filter(TypedDict, total=False):
    messaging_profile_id: str
    """Filter by Messaging Profile ID.

    Use the string `null` for phone numbers without assigned profiles. A synonym for
    the `/messaging_profiles/{id}/short_codes` endpoint when querying about an
    extant profile.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.messaging_hosted_number_orders import action_upload_file_params
from ...types.messaging_hosted_number_orders.action_upload_file_response import ActionUploadFileResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def upload_file(
        self,
        id: str,
        *,
        bill: FileTypes | Omit = omit,
        loa: FileTypes | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUploadFileResponse:
        """
        Upload file required for a messaging hosted number order

        Args:
          bill: Must be the last month's bill with proof of ownership of all of the numbers in
              the order in PDF format.

          loa: Must be a signed LOA for the numbers in the order in PDF format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        body = deepcopy_minimal(
            {
                "bill": bill,
                "loa": loa,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["loa"], ["bill"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/messaging_hosted_number_orders/{id}/actions/file_upload",
            body=maybe_transform(body, action_upload_file_params.ActionUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUploadFileResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def upload_file(
        self,
        id: str,
        *,
        bill: FileTypes | Omit = omit,
        loa: FileTypes | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUploadFileResponse:
        """
        Upload file required for a messaging hosted number order

        Args:
          bill: Must be the last month's bill with proof of ownership of all of the numbers in
              the order in PDF format.

          loa: Must be a signed LOA for the numbers in the order in PDF format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        body = deepcopy_minimal(
            {
                "bill": bill,
                "loa": loa,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["loa"], ["bill"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/messaging_hosted_number_orders/{id}/actions/file_upload",
            body=await async_maybe_transform(body, action_upload_file_params.ActionUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUploadFileResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.upload_file = to_raw_response_wrapper(
            actions.upload_file,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.upload_file = async_to_raw_response_wrapper(
            actions.upload_file,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.upload_file = to_streamed_response_wrapper(
            actions.upload_file,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.upload_file = async_to_streamed_response_wrapper(
            actions.upload_file,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ....._models import BaseModel

__all__ = ["TestSuiteListResponse"]


class TestSuiteListResponse(BaseModel):
    __test__ = False
    data: List[str]
    """Array of unique test suite names available to the user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.metadata import Metadata
from .dynamic_emergency_address import DynamicEmergencyAddress

__all__ = ["DynamicEmergencyAddressListResponse"]


class DynamicEmergencyAddressListResponse(BaseModel):
    data: Optional[List[DynamicEmergencyAddress]] = None

    meta: Optional[Metadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    CustomerServiceRecordListResponse,
    CustomerServiceRecordCreateResponse,
    CustomerServiceRecordRetrieveResponse,
    CustomerServiceRecordVerifyPhoneNumberCoverageResponse,
)
from telnyx._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCustomerServiceRecords:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.create(
            phone_number="+1234567890",
        )
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.create(
            phone_number="+1234567890",
            additional_data={
                "account_number": "123456789",
                "address_line_1": "123 Main St",
                "authorized_person_name": "John Doe",
                "billing_phone_number": "+12065551212",
                "city": "New York",
                "customer_code": "123456789",
                "name": "Entity Inc.",
                "pin": "1234",
                "state": "NY",
                "zip_code": "10001",
            },
            webhook_url="https://example.com/webhook",
        )
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.customer_service_records.with_raw_response.create(
            phone_number="+1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = response.parse()
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.customer_service_records.with_streaming_response.create(
            phone_number="+1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = response.parse()
            assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.retrieve(
            "customer_service_record_id",
        )
        assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.customer_service_records.with_raw_response.retrieve(
            "customer_service_record_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = response.parse()
        assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.customer_service_records.with_streaming_response.retrieve(
            "customer_service_record_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = response.parse()
            assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `customer_service_record_id` but received ''"
        ):
            client.customer_service_records.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.list()
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.list(
            filter={
                "created_at": {
                    "gt": parse_datetime("2020-01-01T00:00:00Z"),
                    "lt": parse_datetime("2020-01-01T00:00:00Z"),
                },
                "phone_number": {
                    "eq": "+12441239999",
                    "in": ["+12441239999"],
                },
                "status": {
                    "eq": "pending",
                    "in": ["pending"],
                },
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort={"value": "created_at"},
        )
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.customer_service_records.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = response.parse()
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.customer_service_records.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = response.parse()
            assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_verify_phone_number_coverage(self, client: Telnyx) -> None:
        customer_service_record = client.customer_service_records.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        )
        assert_matches_type(
            CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_verify_phone_number_coverage(self, client: Telnyx) -> None:
        response = client.customer_service_records.with_raw_response.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = response.parse()
        assert_matches_type(
            CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_verify_phone_number_coverage(self, client: Telnyx) -> None:
        with client.customer_service_records.with_streaming_response.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = response.parse()
            assert_matches_type(
                CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
            )

        assert cast(Any, response.is_closed) is True


class TestAsyncCustomerServiceRecords:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.create(
            phone_number="+1234567890",
        )
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.create(
            phone_number="+1234567890",
            additional_data={
                "account_number": "123456789",
                "address_line_1": "123 Main St",
                "authorized_person_name": "John Doe",
                "billing_phone_number": "+12065551212",
                "city": "New York",
                "customer_code": "123456789",
                "name": "Entity Inc.",
                "pin": "1234",
                "state": "NY",
                "zip_code": "10001",
            },
            webhook_url="https://example.com/webhook",
        )
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.customer_service_records.with_raw_response.create(
            phone_number="+1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = await response.parse()
        assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.customer_service_records.with_streaming_response.create(
            phone_number="+1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = await response.parse()
            assert_matches_type(CustomerServiceRecordCreateResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.retrieve(
            "customer_service_record_id",
        )
        assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.customer_service_records.with_raw_response.retrieve(
            "customer_service_record_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = await response.parse()
        assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.customer_service_records.with_streaming_response.retrieve(
            "customer_service_record_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = await response.parse()
            assert_matches_type(CustomerServiceRecordRetrieveResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `customer_service_record_id` but received ''"
        ):
            await async_client.customer_service_records.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.list()
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.list(
            filter={
                "created_at": {
                    "gt": parse_datetime("2020-01-01T00:00:00Z"),
                    "lt": parse_datetime("2020-01-01T00:00:00Z"),
                },
                "phone_number": {
                    "eq": "+12441239999",
                    "in": ["+12441239999"],
                },
                "status": {
                    "eq": "pending",
                    "in": ["pending"],
                },
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort={"value": "created_at"},
        )
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.customer_service_records.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = await response.parse()
        assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.customer_service_records.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = await response.parse()
            assert_matches_type(CustomerServiceRecordListResponse, customer_service_record, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_verify_phone_number_coverage(self, async_client: AsyncTelnyx) -> None:
        customer_service_record = await async_client.customer_service_records.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        )
        assert_matches_type(
            CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_verify_phone_number_coverage(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.customer_service_records.with_raw_response.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer_service_record = await response.parse()
        assert_matches_type(
            CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_verify_phone_number_coverage(self, async_client: AsyncTelnyx) -> None:
        async with async_client.customer_service_records.with_streaming_response.verify_phone_number_coverage(
            phone_numbers=["+1234567890"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer_service_record = await response.parse()
            assert_matches_type(
                CustomerServiceRecordVerifyPhoneNumberCoverageResponse, customer_service_record, path=["response"]
            )

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting.usage_reports import messaging_list_params, messaging_create_params
from .....types.legacy.reporting.usage_reports.messaging_list_response import MessagingListResponse
from .....types.legacy.reporting.usage_reports.messaging_create_response import MessagingCreateResponse
from .....types.legacy.reporting.usage_reports.messaging_delete_response import MessagingDeleteResponse
from .....types.legacy.reporting.usage_reports.messaging_retrieve_response import MessagingRetrieveResponse

__all__ = ["MessagingResource", "AsyncMessagingResource"]


class MessagingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aggregation_type: int,
        end_time: Union[str, datetime] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingCreateResponse:
        """
        Creates a new legacy usage V2 MDR report request with the specified filters

        Args:
          aggregation_type: Aggregation type: No aggregation = 0, By Messaging Profile = 1, By Tags = 2

          managed_accounts: List of managed accounts to include

          profiles: List of messaging profile IDs to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/usage_reports/messaging",
            body=maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_time": end_time,
                    "managed_accounts": managed_accounts,
                    "profiles": profiles,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "start_time": start_time,
                },
                messaging_create_params.MessagingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Fetch single MDR usage report by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/usage_reports/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """
        Fetch all previous requests for MDR usage reports.

        Args:
          page: Page number

          per_page: Size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/legacy/reporting/usage_reports/messaging",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    messaging_list_params.MessagingListParams,
                ),
            ),
            cast_to=MessagingListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingDeleteResponse:
        """
        Deletes a specific V2 legacy usage MDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/legacy/reporting/usage_reports/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingDeleteResponse,
        )


class AsyncMessagingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aggregation_type: int,
        end_time: Union[str, datetime] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        select_all_managed_accounts: bool | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingCreateResponse:
        """
        Creates a new legacy usage V2 MDR report request with the specified filters

        Args:
          aggregation_type: Aggregation type: No aggregation = 0, By Messaging Profile = 1, By Tags = 2

          managed_accounts: List of managed accounts to include

          profiles: List of messaging profile IDs to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/usage_reports/messaging",
            body=await async_maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_time": end_time,
                    "managed_accounts": managed_accounts,
                    "profiles": profiles,
                    "select_all_managed_accounts": select_all_managed_accounts,
                    "start_time": start_time,
                },
                messaging_create_params.MessagingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingRetrieveResponse:
        """
        Fetch single MDR usage report by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/usage_reports/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingRetrieveResponse,
        )

    async def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingListResponse:
        """
        Fetch all previous requests for MDR usage reports.

        Args:
          page: Page number

          per_page: Size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/legacy/reporting/usage_reports/messaging",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    messaging_list_params.MessagingListParams,
                ),
            ),
            cast_to=MessagingListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingDeleteResponse:
        """
        Deletes a specific V2 legacy usage MDR report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/legacy/reporting/usage_reports/messaging/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingDeleteResponse,
        )


class MessagingResourceWithRawResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.create = to_raw_response_wrapper(
            messaging.create,
        )
        self.retrieve = to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.list = to_raw_response_wrapper(
            messaging.list,
        )
        self.delete = to_raw_response_wrapper(
            messaging.delete,
        )


class AsyncMessagingResourceWithRawResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.create = async_to_raw_response_wrapper(
            messaging.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            messaging.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            messaging.list,
        )
        self.delete = async_to_raw_response_wrapper(
            messaging.delete,
        )


class MessagingResourceWithStreamingResponse:
    def __init__(self, messaging: MessagingResource) -> None:
        self._messaging = messaging

        self.create = to_streamed_response_wrapper(
            messaging.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            messaging.list,
        )
        self.delete = to_streamed_response_wrapper(
            messaging.delete,
        )


class AsyncMessagingResourceWithStreamingResponse:
    def __init__(self, messaging: AsyncMessagingResource) -> None:
        self._messaging = messaging

        self.create = async_to_streamed_response_wrapper(
            messaging.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            messaging.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            messaging.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            messaging.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PhoneNumberAssignmentByProfileRetrieveStatusResponse"]


class PhoneNumberAssignmentByProfileRetrieveStatusResponse(BaseModel):
    status: Literal["pending", "processing", "completed", "failed"]
    """An enumeration."""

    task_id: str = FieldInfo(alias="taskId")

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["CivicAddressListParams", "Filter"]


class CivicAddressListParams(TypedDict, total=False):
    filter: Filter
    """Filter parameter for civic addresses (deepObject style).

    Supports filtering by country.
    """


class Filter(TypedDict, total=False):
    country: SequenceNotStr[str]
    """The country (or countries) to filter addresses by."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from ..porting_orders_activation_job import PortingOrdersActivationJob

__all__ = ["ActivationJobListResponse"]


class ActivationJobListResponse(BaseModel):
    data: Optional[List[PortingOrdersActivationJob]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias, TypedDict

from .aws_voice_settings_param import AwsVoiceSettingsParam
from .transcription_config_param import TranscriptionConfigParam
from .interruption_settings_param import InterruptionSettingsParam
from .telnyx_voice_settings_param import TelnyxVoiceSettingsParam
from .eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam

__all__ = ["ActionStartAIAssistantParams", "Assistant", "VoiceSettings"]


class ActionStartAIAssistantParams(TypedDict, total=False):
    assistant: Assistant
    """AI Assistant configuration"""

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    greeting: str
    """
    Text that will be played when the assistant starts, if none then nothing will be
    played when the assistant starts. The greeting can be text for any voice or SSML
    for `AWS.Polly.<voice_id>` voices. There is a 3,000 character limit.
    """

    interruption_settings: InterruptionSettingsParam
    """Settings for handling user interruptions during assistant speech"""

    transcription: TranscriptionConfigParam
    """The settings associated with speech to text for the voice assistant.

    This is only relevant if the assistant uses a text-to-text language model. Any
    assistant using a model with native audio support (e.g.
    `fixie-ai/ultravox-v0_4`) will ignore this field.
    """

    voice: str
    """The voice to be used by the voice assistant.

    Currently we support ElevenLabs, Telnyx and AWS voices.

    **Supported Providers:**

    - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
      voices, which provide more realistic, human-like speech, append `-Neural` to
      the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
      [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
      for compatibility.
    - **Azure:** Use `Azure.<VoiceId>. (e.g. Azure.en-CA-ClaraNeural,
      Azure.en-CA-LiamNeural, Azure.en-US-BrianMultilingualNeural,
      Azure.en-US-Ava:DragonHDLatestNeural. For a complete list of voices, go to
      [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery).)
    - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
      `ElevenLabs.BaseModel.John`). The `ModelId` part is optional. To use
      ElevenLabs, you must provide your ElevenLabs API key as an integration secret
      under `"voice_settings": {"api_key_ref": "<secret_id>"}`. See
      [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
      for details. Check
      [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
    - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>`
    """

    voice_settings: VoiceSettings
    """The settings associated with the voice selected"""


class Assistant(TypedDict, total=False):
    id: str
    """The identifier of the AI assistant to use"""

    instructions: str
    """
    The system instructions that the voice assistant uses during the start assistant
    command. This will overwrite the instructions set in the assistant
    configuration.
    """

    openai_api_key_ref: str
    """Reference to the OpenAI API key. Required only when using OpenAI models"""


VoiceSettings: TypeAlias = Union[ElevenLabsVoiceSettingsParam, TelnyxVoiceSettingsParam, AwsVoiceSettingsParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .user_bundle import UserBundle
from .pagination_response import PaginationResponse

__all__ = ["UserBundleListResponse"]


class UserBundleListResponse(BaseModel):
    data: List[UserBundle]

    meta: PaginationResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .outbound_message_payload import OutboundMessagePayload

__all__ = ["DeliveryUpdateWebhookEvent", "Data", "Meta"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["message.sent", "message.finalized"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    payload: Optional[OutboundMessagePayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class Meta(BaseModel):
    attempt: Optional[int] = None
    """Number of attempts to deliver the webhook event."""

    delivered_to: Optional[str] = None
    """The webhook URL the event was delivered to."""


class DeliveryUpdateWebhookEvent(BaseModel):
    data: Optional[Data] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.messaging_tollfree.verification import (
    Volume,
    UseCaseCategories,
    TfVerificationStatus,
    TollFreeVerificationEntityType,
    request_list_params,
    request_create_params,
    request_update_params,
)
from ....types.messaging_tollfree.verification.volume import Volume
from ....types.messaging_tollfree.verification.url_param import URLParam
from ....types.messaging_tollfree.verification.use_case_categories import UseCaseCategories
from ....types.messaging_tollfree.verification.request_list_response import RequestListResponse
from ....types.messaging_tollfree.verification.tf_phone_number_param import TfPhoneNumberParam
from ....types.messaging_tollfree.verification.tf_verification_status import TfVerificationStatus
from ....types.messaging_tollfree.verification.verification_request_egress import VerificationRequestEgress
from ....types.messaging_tollfree.verification.verification_request_status import VerificationRequestStatus
from ....types.messaging_tollfree.verification.toll_free_verification_entity_type import TollFreeVerificationEntityType

__all__ = ["RequestsResource", "AsyncRequestsResource"]


class RequestsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RequestsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        isv_reseller: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """
        Submit a new tollfree verification request

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          isv_reseller: ISV name

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messaging_tollfree/verification/requests",
            body=maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "isv_reseller": isv_reseller,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_create_params.RequestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestStatus:
        """
        Get a single verification request by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestStatus,
        )

    def update(
        self,
        id: str,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        isv_reseller: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """Update an existing tollfree verification request.

        This is particularly useful
        when there are pending customer actions to be taken.

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          isv_reseller: ISV name

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/messaging_tollfree/verification/requests/{id}",
            body=maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "isv_reseller": isv_reseller,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_update_params.RequestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    def list(
        self,
        *,
        page: int,
        page_size: int,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        phone_number: str | Omit = omit,
        status: TfVerificationStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequestListResponse:
        """
        Get a list of previously-submitted tollfree verification requests

        Args:
          page_size: Request this many records per page

                      This value is automatically clamped if the provided value is too large.

          status: Tollfree verification status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/messaging_tollfree/verification/requests",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "date_end": date_end,
                        "date_start": date_start,
                        "phone_number": phone_number,
                        "status": status,
                    },
                    request_list_params.RequestListParams,
                ),
            ),
            cast_to=RequestListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a verification request

        A request may only be deleted when when the request is in the "rejected" state.

        - `HTTP 200`: request successfully deleted
        - `HTTP 400`: request exists but can't be deleted (i.e. not rejected)
        - `HTTP 404`: request unknown or already deleted

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncRequestsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRequestsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        isv_reseller: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """
        Submit a new tollfree verification request

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          isv_reseller: ISV name

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messaging_tollfree/verification/requests",
            body=await async_maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "isv_reseller": isv_reseller,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_create_params.RequestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestStatus:
        """
        Get a single verification request by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestStatus,
        )

    async def update(
        self,
        id: str,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        isv_reseller: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """Update an existing tollfree verification request.

        This is particularly useful
        when there are pending customer actions to be taken.

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          isv_reseller: ISV name

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/messaging_tollfree/verification/requests/{id}",
            body=await async_maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "isv_reseller": isv_reseller,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_update_params.RequestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    async def list(
        self,
        *,
        page: int,
        page_size: int,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        phone_number: str | Omit = omit,
        status: TfVerificationStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequestListResponse:
        """
        Get a list of previously-submitted tollfree verification requests

        Args:
          page_size: Request this many records per page

                      This value is automatically clamped if the provided value is too large.

          status: Tollfree verification status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/messaging_tollfree/verification/requests",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "date_end": date_end,
                        "date_start": date_start,
                        "phone_number": phone_number,
                        "status": status,
                    },
                    request_list_params.RequestListParams,
                ),
            ),
            cast_to=RequestListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a verification request

        A request may only be deleted when when the request is in the "rejected" state.

        - `HTTP 200`: request successfully deleted
        - `HTTP 400`: request exists but can't be deleted (i.e. not rejected)
        - `HTTP 404`: request unknown or already deleted

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class RequestsResourceWithRawResponse:
    def __init__(self, requests: RequestsResource) -> None:
        self._requests = requests

        self.create = to_raw_response_wrapper(
            requests.create,
        )
        self.retrieve = to_raw_response_wrapper(
            requests.retrieve,
        )
        self.update = to_raw_response_wrapper(
            requests.update,
        )
        self.list = to_raw_response_wrapper(
            requests.list,
        )
        self.delete = to_raw_response_wrapper(
            requests.delete,
        )


class AsyncRequestsResourceWithRawResponse:
    def __init__(self, requests: AsyncRequestsResource) -> None:
        self._requests = requests

        self.create = async_to_raw_response_wrapper(
            requests.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            requests.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            requests.update,
        )
        self.list = async_to_raw_response_wrapper(
            requests.list,
        )
        self.delete = async_to_raw_response_wrapper(
            requests.delete,
        )


class RequestsResourceWithStreamingResponse:
    def __init__(self, requests: RequestsResource) -> None:
        self._requests = requests

        self.create = to_streamed_response_wrapper(
            requests.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            requests.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            requests.update,
        )
        self.list = to_streamed_response_wrapper(
            requests.list,
        )
        self.delete = to_streamed_response_wrapper(
            requests.delete,
        )


class AsyncRequestsResourceWithStreamingResponse:
    def __init__(self, requests: AsyncRequestsResource) -> None:
        self._requests = requests

        self.create = async_to_streamed_response_wrapper(
            requests.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            requests.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            requests.update,
        )
        self.list = async_to_streamed_response_wrapper(
            requests.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            requests.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["OtaUpdateRetrieveResponse", "Data", "DataSettings", "DataSettingsMobileNetworkOperatorsPreference"]


class DataSettingsMobileNetworkOperatorsPreference(BaseModel):
    mobile_network_operator_id: Optional[str] = None
    """The mobile network operator resource identification UUID."""

    mobile_network_operator_name: Optional[str] = None
    """The mobile network operator resource name."""

    priority: Optional[int] = None
    """
    It determines what is the priority of a specific network operator that should be
    assumed by a SIM card when connecting to a network. The highest priority is 0,
    the second highest is 1 and so on.
    """


class DataSettings(BaseModel):
    mobile_network_operators_preferences: Optional[List[DataSettingsMobileNetworkOperatorsPreference]] = None
    """
    A list of mobile network operators and the priority that should be applied when
    the SIM is connecting to the network.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    settings: Optional[DataSettings] = None
    """A JSON object representation of the operation.

    The information present here will relate directly to the source of the OTA
    request.
    """

    sim_card_id: Optional[str] = None
    """The identification UUID of the related SIM card resource."""

    status: Optional[Literal["in-progress", "completed", "failed"]] = None

    type: Optional[Literal["sim_card_network_preferences"]] = None
    """Represents the type of the operation requested.

    This will relate directly to the source of the request.
    """

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""


class OtaUpdateRetrieveResponse(BaseModel):
    data: Optional[Data] = None
    """This object represents an Over the Air (OTA) update request.

    It allows tracking the current status of a operation that apply settings in a
    particular SIM card. <br/><br/>
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .verification_request_status import VerificationRequestStatus

__all__ = ["RequestListResponse"]


class RequestListResponse(BaseModel):
    records: Optional[List[VerificationRequestStatus]] = None
    """The records yielded by this request"""

    total_records: Optional[int] = None
    """The total amount of records for these query parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    UserAddressListResponse,
    UserAddressCreateResponse,
    UserAddressRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUserAddresses:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        user_address = client.user_addresses.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        )
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        user_address = client.user_addresses.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
            administrative_area="TX",
            borough="Guadalajara",
            customer_reference="MY REF 001",
            extended_address="14th Floor",
            neighborhood="Ciudad de los deportes",
            phone_number="+12125559000",
            postal_code="78701",
            skip_address_verification="skip_address_verification",
        )
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.user_addresses.with_raw_response.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = response.parse()
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.user_addresses.with_streaming_response.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = response.parse()
            assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        user_address = client.user_addresses.retrieve(
            "id",
        )
        assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.user_addresses.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = response.parse()
        assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.user_addresses.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = response.parse()
            assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.user_addresses.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        user_address = client.user_addresses.list()
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        user_address = client.user_addresses.list(
            filter={
                "customer_reference": {
                    "contains": "contains",
                    "eq": "eq",
                },
                "street_address": {"contains": "contains"},
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="street_address",
        )
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.user_addresses.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = response.parse()
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.user_addresses.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = response.parse()
            assert_matches_type(UserAddressListResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUserAddresses:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        user_address = await async_client.user_addresses.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        )
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_address = await async_client.user_addresses.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
            administrative_area="TX",
            borough="Guadalajara",
            customer_reference="MY REF 001",
            extended_address="14th Floor",
            neighborhood="Ciudad de los deportes",
            phone_number="+12125559000",
            postal_code="78701",
            skip_address_verification="skip_address_verification",
        )
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.user_addresses.with_raw_response.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = await response.parse()
        assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.user_addresses.with_streaming_response.create(
            business_name="Toy-O'Kon",
            country_code="US",
            first_name="Alfred",
            last_name="Foster",
            locality="Austin",
            street_address="600 Congress Avenue",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = await response.parse()
            assert_matches_type(UserAddressCreateResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        user_address = await async_client.user_addresses.retrieve(
            "id",
        )
        assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.user_addresses.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = await response.parse()
        assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.user_addresses.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = await response.parse()
            assert_matches_type(UserAddressRetrieveResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.user_addresses.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        user_address = await async_client.user_addresses.list()
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_address = await async_client.user_addresses.list(
            filter={
                "customer_reference": {
                    "contains": "contains",
                    "eq": "eq",
                },
                "street_address": {"contains": "contains"},
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="street_address",
        )
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.user_addresses.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_address = await response.parse()
        assert_matches_type(UserAddressListResponse, user_address, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.user_addresses.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_address = await response.parse()
            assert_matches_type(UserAddressListResponse, user_address, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionValidateRegistrationCodesParams"]


class ActionValidateRegistrationCodesParams(TypedDict, total=False):
    registration_codes: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["LedgerBillingGroupReportCreateParams"]


class LedgerBillingGroupReportCreateParams(TypedDict, total=False):
    month: int
    """Month of the ledger billing group report"""

    year: int
    """Year of the ledger billing group report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = [
    "UploadListParams",
    "Filter",
    "FilterCivicAddressID",
    "FilterLocationID",
    "FilterPhoneNumber",
    "FilterStatus",
    "Page",
]


class UploadListParams(TypedDict, total=False):
    filter: Filter
    """Filter parameter for uploads (deepObject style).

    Supports filtering by status, civic_address_id, location_id, and phone_number
    with eq/contains operations.
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterCivicAddressID(TypedDict, total=False):
    eq: str
    """The civic address ID to filter by"""


class FilterLocationID(TypedDict, total=False):
    eq: str
    """The location ID to filter by"""


class FilterPhoneNumber(TypedDict, total=False):
    contains: str
    """The phone number to filter by (partial match)"""

    eq: str
    """The phone number to filter by (exact match)"""


class FilterStatus(TypedDict, total=False):
    eq: List[Literal["pending_upload", "pending", "in_progress", "success", "error"]]
    """The status of the upload to filter by"""


class Filter(TypedDict, total=False):
    civic_address_id: FilterCivicAddressID

    location_id: FilterLocationID

    phone_number: FilterPhoneNumber

    status: FilterStatus


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

from .video_region_param import VideoRegionParam

__all__ = ["RoomCompositionCreateParams"]


class RoomCompositionCreateParams(TypedDict, total=False):
    format: Optional[str]
    """The desired format of the room composition."""

    resolution: Optional[str]
    """
    The desired resolution (width/height in pixels) of the resulting video of the
    room composition. Both width and height are required to be between 16 and 1280;
    and width _ height should not exceed 1280 _ 720
    """

    session_id: Optional[str]
    """id of the room session associated with the room composition."""

    video_layout: Dict[str, VideoRegionParam]
    """Describes the video layout of the room composition in terms of regions."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this room composition will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this room composition will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["GoogleTranscriptionLanguage"]

GoogleTranscriptionLanguage: TypeAlias = Literal[
    "af",
    "sq",
    "am",
    "ar",
    "hy",
    "az",
    "eu",
    "bn",
    "bs",
    "bg",
    "my",
    "ca",
    "yue",
    "zh",
    "hr",
    "cs",
    "da",
    "nl",
    "en",
    "et",
    "fil",
    "fi",
    "fr",
    "gl",
    "ka",
    "de",
    "el",
    "gu",
    "iw",
    "hi",
    "hu",
    "is",
    "id",
    "it",
    "ja",
    "jv",
    "kn",
    "kk",
    "km",
    "ko",
    "lo",
    "lv",
    "lt",
    "mk",
    "ms",
    "ml",
    "mr",
    "mn",
    "ne",
    "no",
    "fa",
    "pl",
    "pt",
    "pa",
    "ro",
    "ru",
    "rw",
    "sr",
    "si",
    "sk",
    "sl",
    "ss",
    "st",
    "es",
    "su",
    "sw",
    "sv",
    "ta",
    "te",
    "th",
    "tn",
    "tr",
    "ts",
    "uk",
    "ur",
    "uz",
    "ve",
    "vi",
    "xh",
    "zu",
]
